
#########################
# CODE 
# R/QTL - HK model
# F2 intercross
# August 2010
########################

rm(list=ls())
library(qtl)
library(qtlbook)
library(rlecuyer)


###############################
#       load csv data         #
###############################


QTL351 = read.cross(format="csv", file="/Users/christophersauvage/Documents/PostDoc/QTL/RQTL/N=351/Analyses_QTL_351_all.csv", genotypes=c("AA", "AB", "BB", "1", "2", "3", "4"), na.strings=c("NA"), convertXdata=TRUE)

QTL351 = jittermap(QTL351) 


##############################
#      Compute GenoProb      #
##############################

QTL351 = calc.genoprob(raw.data, step = 1, error.prob =0.001)

QTL351 = calc.genoprob(raw.data, step=2.0, off.end=0.0, error.prob=1.0E-4, map.function="kosambi", stepwidth="fixed")

summary(QTL351)


###############################
#        Check dataset        #
###############################

nind (QTL351)
nphe(QTL351)
totmar(QTL351)
nchr(QTL351)
nmar(QTL351)

plot.cross(QTL351)

#plot.missing(QTL351)
#plot.map(QTL351, show.marker.names=F)
#plot.pheno(QTL351)


###############################
#       Check phenotype       #
###############################



# cross check of phenotype data
###########

n.pheno = 28

par(mfrow=c(5,6))
for(i in 1:n.pheno) 

{
	plot.pheno(QTL351, pheno.col=i)

}

pairs(jitter( as.matrix(raw.data$pheno)), cex=0.6, las=1)

# cex = size of points
# las = orientation of y axis labels


##
# plot the individuals phenotypes against their index
##############

par(mfrow=c(5,5), las=1, cex=0.8)
means = apply(raw.data$pheno, 1, mean)
plot(means)
plot(sample(means), xlab="Random index", ylab="means")


##
# segregation distortion
#############

gt = geno.table(raw.data)
gt[ gt$P.value < 1e-7, ]

write.table(gt, "distortion_segregation_N351.txt", sep="\t", col.names = T, row.names=T)


##
# Compare individuals' genotypes
#############

cg = comparegeno(raw.data)
hist(cg, breaks=200, xlab="Proportion of identical genotypes")
rug(cg)

which(cg > 0.9, arr.ind=TRUE)

##
# Pairwise recombination fractions
##############

rec.frac = est.rf(raw.data)

checkAlleles(raw.data)

plot.rf(raw.data, alternate.chrid=TRUE)



##
# Counting crossover
##############

#nxo = countXO(raw.data)
#plot(nxo, ylab = 'No. crossovers")


#################################
#								#
#   Genome scan for one trait   #
#								#
#################################


n.pheno = 28
out.qtl.all.pheno = NULL


for(i in 1:n.pheno) 

{
	out.qtl.all.pheno= rbind(out.qtl.all.pheno, scanone(raw.data, pheno.col=i, model = "normal", method = "hk", n.cluster = 4))
	
}

summary(out.qtl.all.pheno)

#write.table(summary(out.qtl.all.pheno), "summary_out.qtl.all.pheno_N206.txt", sep="\t", col.names = T, row.names=T)

operm = scanone(raw.data, n.perm=10000, verbose=FALSE, n.cluster=4)

summary(out.qtl.all.pheno, perms= operm, alpha = 0.05, pvalues = TRUE)


#################


out.hk1 = scanone(raw.data, pheno.col=1, model = "normal", method = "hk", n.cluster = 4)
out.hk2 = scanone(raw.data, pheno.col=2, model = "normal", method = "hk", n.cluster = 4)
out.hk3 = scanone(raw.data, pheno.col=3, model = "normal", method = "hk", n.cluster = 4)
out.hk4 = scanone(raw.data, pheno.col=4, model = "normal", method = "hk", n.cluster = 4)

par(mfrow=c(4,1))

plot(out.hk1, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Poids des individus (Mai 2009)" )
abline(h=3, lty=3)
plot(out.hk2, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Poids en Juillet")
abline(h=3, lty=3)
plot(out.hk3, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Poids en Aout")
abline(h=3, lty=3)
plot(out.hk4, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Poids en Novembre")
abline(h=3, lty=3)




out.hk5 = scanone(raw.data, pheno.col=5, model = "normal", method = "hk", n.cluster = 4)
out.hk6 = scanone(raw.data, pheno.col=6, model = "normal", method = "hk", n.cluster = 4)
out.hk7 = scanone(raw.data, pheno.col=7, model = "normal", method = "hk", n.cluster = 4)
out.hk8 = scanone(raw.data, pheno.col=8, model = "normal", method = "hk", n.cluster = 4)


par(mfrow=c(4,1))

plot(out.hk5, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Taille en Mai")
abline(h=3, lty=3)
plot(out.hk6, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Taille en Juillet")
abline(h=7, lty=3)
plot(out.hk7, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Taille en Aout")
abline(h=3, lty=3)
plot(out.hk8, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Taille en Novembre")
abline(h=3, lty=3)




out.hk9 = scanone(raw.data, pheno.col=9, model = "normal", method = "hk", n.cluster = 4)
out.hk10 = scanone(raw.data, pheno.col=10, model = "normal", method = "ehk", n.cluster = 4)
out.hk11 = scanone(raw.data, pheno.col=11, model = "normal", method = "hk", n.cluster = 4)
out.hk12 = scanone(raw.data, pheno.col=12, model = "normal", method = "hk", n.cluster = 4)

par(mfrow=c(4,1))

plot(out.hk9, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Facteur de condition en Mai")
abline(h=3, lty=3)
plot(out.hk10, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Facteur de condition en Juillet")
abline(h=3, lty=3)
plot(out.hk11, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Facteur de condition en Aout")
abline(h=3, lty=3)
plot(out.hk12, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Facteur de condition en Novembre")
abline(h=3, lty=3)



out.hk13 = scanone(raw.data, pheno.col=13, model = "normal", method = "hk", n.cluster = 4)
out.hk14 = scanone(raw.data, pheno.col=14, model = "normal", method = "hk", n.cluster = 4)
out.hk15 = scanone(raw.data, pheno.col=15, model = "normal", method = "hk", n.cluster = 4)


par(mfrow=c(3,1))

plot(out.hk13, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Poids humide du foie")
abline(h=3, lty=3)
plot(out.hk14, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Indice Hpato-somatique")
abline(h=3, lty=3)
plot(out.hk15, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Hmatocrite")
abline(h=3, lty=3)


out.hk16 = scanone(raw.data, pheno.col=16, model = "normal", method = "hk", n.cluster = 4)
out.hk17 = scanone(raw.data, pheno.col=17, model = "normal", method = "hk", n.cluster = 4)
out.hk18 = scanone(raw.data, pheno.col=18, model = "normal", method = "hk", n.cluster = 4)
out.hk19 = scanone(raw.data, pheno.col=19, model = "normal", method = "hk", n.cluster = 4)
out.hk20 = scanone(raw.data, pheno.col=20, model = "normal", method = "hk", n.cluster = 4)
out.hk21 = scanone(raw.data, pheno.col=21, model = "normal", method = "hk", n.cluster = 4)
out.hk22 = scanone(raw.data, pheno.col=22, model = "normal", method = "hk", n.cluster = 4)



par(mfrow=c(4,1))

plot(out.hk16, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "18S (gene reference)")
abline(h=3, lty=3)
plot(out.hk17, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "GHR")
abline(h=3, lty=3)
plot(out.hk18, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "IGF-1")
abline(h=3, lty=3)
plot(out.hk19, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "IGF1-R")
abline(h=3, lty=3)

par(mfrow=c(3,1))

plot(out.hk20, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "EF1")
abline(h=3, lty=3)
plot(out.hk21, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "B-actine")
abline(h=3, lty=3)
plot(out.hk22, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "sexe des animaux")
abline(h=3, lty=3)


out.hk23 = scanone(raw.data, pheno.col=23, model = "normal", method = "hk", n.cluster = 4)
out.hk24 = scanone(raw.data, pheno.col=24, model = "normal", method = "hk", n.cluster = 4)
out.hk25 = scanone(raw.data, pheno.col=25, model = "normal", method = "hk", n.cluster = 4)
out.hk26 = scanone(raw.data, pheno.col=26, model = "normal", method = "hk", n.cluster = 4)
out.hk27 = scanone(raw.data, pheno.col=27, model = "normal", method = "hk", n.cluster = 4)
out.hk28 = scanone(raw.data, pheno.col=28, model = "normal", method = "hk", n.cluster = 4)


par(mfrow=c(3,1))

plot(out.hk23, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "delta_cortisol")
abline(h=3, lty=3)
plot(out.hk24, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "delta_osmo")
abline(h=3, lty=3)
plot(out.hk25, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "delta_chlore")
abline(h=3, lty=3)

par(mfrow=c(3,1))

plot(out.hk26, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "diam_oeufs")
abline(h=3, lty=3)
plot(out.hk27, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "conc_sperm")
abline(h=3, lty=3)
plot(out.hk28, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "diam_sperm")
abline(h=3, lty=3)



#####################

out.qtl.all.pheno.unique = cbind(out.hk1, out.hk2, out.hk3)	

summary(out.qtl.all.pheno.unique)

#summary(out.qtl.all.pheno.unique, alpha = 0.05, pvalues = TRUE)

####################
# "Genomewide level of significance"
####################################

out.hk2 = scanone(raw.data, pheno.col=1, model = "normal", method = "hk")

plot(out.hk2, chr=c(1:39), lty = 1, col = c("blue"), xlab = "LG", ylab = "Lod score", main = "Genome scan of Lod Score for phenotype = Taille")
abline(h=3, lty=3)



######################################
# HK method with multiple phenotypes #
######################################

outall.hk = scanone(raw.data, pheno.col=1:22, model = "normal", method = "hk", n.cluster = 4)

summary(outall.hk)

write.table(summary(outall.hk), "summary_outall.hk_N206.txt", sep="\t", col.names = T, row.names=T)


#par(mfrow=c(3,1))
#plot(outall.hk, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "")
#abline(h=3, lty=3)



##
# Significance threshold
#############

operm = scanone(QTL351, n.perm=10000, verbose=FALSE, n.cluster=4)

plot(operm)

# genomewide level of significance for 20% and 5%

summary(operm, alpha=c(0.20, 0.05))

# Genome-scan adjusted p values for each LOD peak -> (10%)

summary(outall.hk, perms=operm, alpha=0.5, pvalues=TRUE)

# Confidence level on LOD

binom.test(0, 10000)$conf.int


##
# Interval estimates of QTL position
##############

bayesint(outall.hk, 5, 0.95, expandtomarkers=T)

##
# Boostrap based confidence interval
##############

out.boot = scanoneboot(raw.data, chr = 1:40, n.boot = 10000)

summary(out.boot)

# write.table(summary(out.boot), "Summary_outboot_N351.txt", sep="\t", col.names = T, row.names=T)



##
# Histogram of the estimated QTL locations in 10000 bootstrap replicates
######################################################################

par(mar=c(4.1,0.1,0.1,0.1),las=1)
hist(out.boot, main="Bootstrap values of QTL position", xlab="Estimated QTL location (cM)",
     ylab="", yaxt="n")


rug(raw.data$geno[[5]]$map)


##
# Multiple phenotypes (give position, Lod and pvalues)
##########

summary(outall.hk, format = "allpeaks", perms = operm, alpha = 0.05, pvalues = TRUE)




#################################
#								#
#   Genome scan '2 part model'  #
#								#
#################################

# nchr = 19

out.hk = scanone(raw.data, pheno.col=(1:3), model = "2part", method = "hk")

outperm.ehk = scanone(raw.data, pheno.col = (1:3), model = "2part", upper = TRUE, n.perm = 5)

summary(out.ehk, perms = outperm.ehk, alpha = 0.05, pvalues = TRUE, format = "allpeaks")




par(mfrow=c(2,1))

plot(out.ehk, chr=c(1:40), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Genome scan of Lod Score for phenotype = Poids")

abline(h=3, lty=3)

#################################
#								#
#   Genome scan '2 part model'  #
#	 Sex Specific				#
#								#
#################################


##
# Male
########

outmale.ehk = scanone(subset(raw.data, ind  = sex==1), pheno.col = 3, model = "normal", method = "ehk")

plot(outmale.ehk, chr=c(1:19), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Genome scan of Lod Score for phenotype = Poids") 


##
# Female
#########

outfemale.ehk = scanone(subset(raw.data, ind  = sex==0), pheno.col = 1, model = "normal", method = "ehk")

plot(outfemale.ehk, chr=c(1:19), lty = 1, col = c("red"), xlab = "LG", ylab = "Lod score", main = "Genome scan of Lod Score for phenotype = Poids") 


#################################
#								#
#   Genome scan '2 QTL model'   #
#    interaction QTL x QTL		#
#								#
#################################



data = calc.genoprob(raw.data, step = 0, error.prob=0.001, map.function = "kosambi")

time.begin=Sys.time()

out.2 = scantwo(data, verbose = FALSE, n.cluster=4)

summary.scantwo(out.2)

time.finish=Sys.time()
time.total2=difftime(time.finish, time.begin)
time.total2

summary(out.2)

write.table(summary.scantwo(out.2), "summary.scantwo(out2)_N206.txt", sep="\t", col.names = T, row.names=T)

plot(out.2, chr=c(1:40), upper = "cond-add", lower = "cond-int")




##
# Multiple Phenotpes - Two QTLs model
#####################################


data = calc.genoprob(raw.data, step = 0, error.prob=0.001, map.function = "kosambi")


n.pheno = 28
out.2 = NULL


time.begin=Sys.time()

for(i in 1:n.pheno) 

{
	out.2 = rbind(out.2, scantwo(data, verbose = FALSE, n.cluster=4, pheno.col=i, model = "normal", method = "hk"))

}



summary.scantwo(out.2)

time.finish=Sys.time()
time.total2=difftime(time.finish, time.begin)
time.total2


#############

n.pheno = 22

for(i in 1:n.pheno) 

{
	out.qtl.all.pheno= rbind(out.qtl.all.pheno, scanone(raw.data, pheno.col=i, model = "normal", method = "hk", n.cluster = 4))
	
}




#####################
#                   #
# QTL effects - PVE #
#                   #
#####################

##
# makeqtl
#########

# creates a qtl object wit positions from the scan summary

makeqtl = sim.geno(raw.data, step = 2, n.draws = 128, err = 0.01)

qtl = makeqtl(makeqtl, chr = c(1:40), pos = c(68.3, 30, 30, 18))


##########################################################################
#  Locations of the QTL object qtl on the genetic map for the hyper data #
##########################################################################

par(mar=c(4.1,4.1,0.1,0.1), cex=0.7, cex.axis=1.2, cex.lab=1.2)
plot(qtl, main="")





##
# fitqtl
##########

# fit a qtl model depending on the scan results

# Usage : 

fitqtl(cross, pheno.col=1, qtl, covar=NULL, formula, method=c("imp", "hk"), dropone=TRUE, get.ests=FALSE, run.checks=TRUE)

refineqtl



###############################
# Compute confidence interval #
###############################

Interval = lodint(results, chr, qtl.index, drop=1, lodcolumn=1, expandtomarkers=FALSE)


Interval = lodint(QTL35192177.QTLfitted_oestradiol, chr=c("4", "9"), drop=1)
Interval


#bayesint(results, chr, qtl.index, prob=0.95, lodcolumn=1, expandtomarkers=FALSE)




###################################
# Plot phenotype against genotype #
###################################

# usage : find.marker(cross, chr, pos, index)

mar1 = find.marker(QTL351, c(3), pos = c(44.8)) 

mar1


write.table(mar1, "mar1_IGF1R.txt", sep="\t", col.names = T, row.names=T)


# usage : plot.pxg(x, marker, pheno.col=1, jitter=1, infer=TRUE, pch, ylab, main, col, ...)


par(mfrow=c(3,2))
plot.pxg(QTL351, marker= mar1, pheno.col= 1, infer = TRUE, jitter=1, col=2, main = "Poids")



# Autre visualisation de l'effet des Allles au QTL
#
# usage : effectplot(cross, pheno.col=1, mname1, mark1, geno1, mname2, mark2,
#           geno2, main, ylim, xlab, ylab, col, add.legend=TRUE,
#           legend.lab, draw=TRUE, var.flag=c("pooled","group"))
#
# par(mfrow=c(2,1))
#


effectplot(QTL351, pheno.col = 1, "31@0", add.legend=FALSE)


###############################################
#   Estimation additive and dominance effects #
###############################################


# usage : Geno = data.frame(cross$geno[[chromosome.number]]$prob[,"marker.name",])

Geno = NULL
Geno = data.frame(QTL351$geno[[3]]$prob[,"OMM5008",])

Geno$a = (Geno$AC - Geno$BD)/2
Geno$d = (Geno$BC + Geno$AD-Geno$AC-Geno$BD)/2
Geno$i = (Geno$BC - Geno$AD)/2

########################
# Estimation a, d et i #
########################

#testmod = NULL

#testmod = glm(QTL351$pheno$IGF1/100 ~ a + d + i, data=Geno)

#summary(testmod)

###################
# Allelic effects #
###################

#QTL351 = sim.geno(QTL351, map.function="kosambi", stepwidth="fixed")

qtl=NULL
qtl = makeqtl(QTL351, chr =  c(1), pos = c(70.5))
qtl


fitted_qtl = fitqtl(QTL351, pheno.col=1, qtl, covar = NULL, formula = "y~Q1", method = c("imp"), dropone = FALSE, get.ests = TRUE)





####

#qtl = NULL
#qtl = makeqtl(QTL351, chr = c(3), pos = c(78.2))
#qtl
#
#fitted_qtl = fitqtl(QTL351, pheno.col=2, qtl, covar = NULL, formula = "y~Q1", method = c("imp"), dropone = FALSE, get.ests = TRUE)



ClMeans = c(0,0,0,0)
ClSe = c(0,0,0,0)
names(ClMeans) = c("AC","BC","AD","BD")

Gmean = NULL

Gmean = fitted_qtl$ests$ests

Gmean

dim(Gmean) = c(7,1)
ClMeans[1] = Gmean[1]
ClMeans[2] = sum(Gmean[c(1,2)])
ClMeans[3] = sum(Gmean[c(1,3)])
ClMeans[4] = sum(Gmean[c(1,4)])
ClMeans



ad = c(0,0)
names(adi) = c("2a","d")
ad[1] = ClMeans[1]-ClMeans[4]
ad[2] = (ClMeans[2]+ClMeans[3]- ClMeans[1]-ClMeans[4])/2
ad



###
# Loading data
################



data = read.table("QTL_list.txt", sep="\t", header = T, stringsAsFactors = F)

dim(data)

caractere = data[,1]
chr = data[,2]
pos = data[,3]
marker = data[,4]



###
# loop GLM to get SE
####################


Geno = NULL
testmod = NULL
summary.se = NULL


for(j in 5:ncol(QTL351))

{

	Geno = data.frame(QTL351$geno[[chr[j]]]$prob[,marker[j],])

	Geno$a = (Geno$AC - Geno$BD)/2
	Geno$d = (Geno$BC + Geno$AD-Geno$AC-Geno$BD)/2
#	Geno$i = (Geno$BC - Geno$AD)/2

	testmod = glm(QTL351$pheno$oeufs ~ a + d , data=Geno)

	summary.se = cbind(summary.se, summary(testmod)$coefficients[,2])


}

summary.se

write.table(summary.se, "summary_se_oeufs.txt", sep="\t", col.names = T, row.names=T)


